﻿#region License Information

/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

#endregion
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.DataAnalysis {
  [StorableType("A56BFB05-8F11-4766-9FBF-20C7010F1CA3")]
  [Item("ShapeConstraints", "Represents shape constraints associated with a regression problem data e.g. monotonicity constraints.")]
  public class ShapeConstraints : ItemCollection<ShapeConstraint> {
    // private static readonly string exampleInput = "# Example for a target variable constraint:" + Environment.NewLine +
    //                                               "f in [0 .. 100]" + Environment.NewLine + Environment.NewLine +
    //                                               "# Example for constraints on model parameters: " + Environment.NewLine +
    //                                               "df/d'x' in [0 .. 10]" + Environment.NewLine +
    //                                               "∂²f/∂'x'² in [-1 .. inf.]";
    // 
    // [Storable] 
    // private string input;

    // public string Input {
    //   get => input;
    //   set {
    //     if (input == value) return;
    //     input = value;
    //     OnChanged();
    //   }
    // }

    // [Storable] 
    // private string infoText;

    // public string InfoText {
    //   get => infoText;
    //   set {
    //     if (infoText == value) return;
    //     infoText = value;
    //     OnChanged();
    //   }
    // }
    // 
    // [Storable] 
    // private Color infoColor;
    // 
    // public Color InfoColor {
    //   get => infoColor;
    //   set {
    //     if (infoColor == value) return;
    //     infoColor = value;
    //     OnChanged();
    //   }
    // }

    // [Storable] 
    // private IEnumerable<IntervalConstraint> constraints;

    // public IEnumerable<IntervalConstraint> Constraints {
    //   get => constraints;
    //   set {
    //     if (constraints == value) return;
    //     constraints = value.ToList();
    //     OnChanged();
    //   }
    // }

    public IEnumerable<ShapeConstraint> EnabledConstraints => base.list.Where(c => c.Enabled);

    protected override void OnItemsAdded(IEnumerable<ShapeConstraint> items) {
      base.OnItemsAdded(items);
      foreach (var item in items)
        item.Changed += Item_Changed;
    }

    protected override void OnItemsRemoved(IEnumerable<ShapeConstraint> items) {
      base.OnItemsRemoved(items);
      foreach (var item in items)
        item.Changed -= Item_Changed;
    }

    protected override void OnCollectionReset(IEnumerable<ShapeConstraint> items, IEnumerable<ShapeConstraint> oldItems) {
      base.OnCollectionReset(items, oldItems);
      foreach (var item in items)
        item.Changed += Item_Changed;
      foreach (var item in oldItems)
        item.Changed -= Item_Changed;
    }

    private void Item_Changed(object sender, EventArgs e) {
      RaiseChanged();
    }

    // [Storable] 
    // public IRegressionProblemData ProblemData { get; private set; }

    [StorableConstructor]
    protected ShapeConstraints(StorableConstructorFlag _) : base(_) { }

    protected ShapeConstraints(ShapeConstraints original, Cloner cloner)
      : base(original, cloner) {
      // Input = original.Input;
      // InfoText = original.InfoText;
      // infoColor = original.InfoColor;
      // constraints = original.Constraints.Select(cloner.Clone).ToList();
      // ProblemData = cloner.Clone(original.ProblemData);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ShapeConstraints(this, cloner);
    }

    public ShapeConstraints() {
      // Input = exampleInput;
      // InfoText = "";
      // InfoColor = Color.DarkOrange;
      // constraints = new List<IntervalConstraint>();
      // ProblemData = null;
    }

    // public ShapeConstraints(IRegressionProblemData problemData) {
    //   Input = exampleInput;
    //   InfoText = infoText;
    //   InfoColor = InfoColor;
    //   constraints = new List<IntervalConstraint>();
    //   ProblemData = problemData;
    // }

    public event EventHandler Changed;

    private void RaiseChanged() {
      var handlers = Changed;
      if (handlers != null)
        handlers(this, EventArgs.Empty);
    }
  }
}